package com.ruoyi.project.affair.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.affair.domain.TOfficesupply;
import com.ruoyi.project.affair.service.ITOfficesupplyService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 办公用品管理Controller
 *
 * @author ruoyi
 * @date 2020-12-02
 */
@RestController
@RequestMapping("/affair/officesupply")
public class TOfficesupplyController extends BaseController
{
    @Autowired
    private ITOfficesupplyService tOfficesupplyService;

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    /**
     * 查询办公用品管理列表
     */
    @PreAuthorize("@ss.hasPermi('affair:officesupply:list')")
    @GetMapping("/list")
    public TableDataInfo list(TOfficesupply tOfficesupply)
    {
        startPage();
        List<TOfficesupply> list = tOfficesupplyService.selectTOfficesupplyList(tOfficesupply);
        return getDataTable(list);
    }

    /**
     * 导出办公用品管理列表
     */
    @PreAuthorize("@ss.hasPermi('affair:officesupply:export')")
    @Log(title = "办公用品管理", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TOfficesupply tOfficesupply)
    {
        List<TOfficesupply> list = tOfficesupplyService.selectTOfficesupplyList(tOfficesupply);
        ExcelUtil<TOfficesupply> util = new ExcelUtil<TOfficesupply>(TOfficesupply.class);
        return util.exportExcel(list, "officesupply");
    }

    /**
     * 获取办公用品管理详细信息
     */
    @PreAuthorize("@ss.hasPermi('affair:officesupply:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(tOfficesupplyService.selectTOfficesupplyById(id));
    }

    /**
     * 新增办公用品管理
     */
    @PreAuthorize("@ss.hasPermi('affair:officesupply:add')")
    @Log(title = "办公用品管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TOfficesupply tOfficesupply)
    {
        tOfficesupply.setCreaterCode(getUserId().toString());
        return toAjax(tOfficesupplyService.insertTOfficesupply(tOfficesupply));
    }

    /**
     * 修改办公用品管理
     */
    @PreAuthorize("@ss.hasPermi('affair:officesupply:edit')")
    @Log(title = "办公用品管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TOfficesupply tOfficesupply)
    {
        tOfficesupply.setUpdaterCode(getUserId().toString());
        tOfficesupply.setUpdatedate(new Date());
        return toAjax(tOfficesupplyService.updateTOfficesupply(tOfficesupply));
    }

    /**
     * 删除办公用品管理
     */
    @PreAuthorize("@ss.hasPermi('affair:officesupply:remove')")
    @Log(title = "办公用品管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(tOfficesupplyService.deleteTOfficesupplyByIds(ids));
    }

    /**
     * 批量导入办公用品管理
     */
    @PreAuthorize("@ss.hasPermi('affair:officesupply:add')")
    @Log(title = "办公用品管理", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TOfficesupply> list = new ArrayList<TOfficesupply>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TOfficesupply entity = new TOfficesupply();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        if (cellValue != ""){
                            entity.setSerialnumber(Long.parseLong(cellValue));//序号
                        }
                    } else if (j == 2) {
                        entity.setBrand(cellValue);//品牌
                    } else if (j == 3) {
                        entity.setName(cellValue);//商品名称
                    } else if (j == 4) {
                        entity.setSpecs(cellValue);//规格颜色
                    } else if (j == 5) {
                        entity.setGoodsno(cellValue);//货号
                    } else if (j == 6) {
                        entity.setAddress(cellValue);//产地
                    } else if (j == 7) {
                        entity.setUnit(cellValue);//单元
                    } else if (j == 8) {
                        if (cellValue != ""){
                            entity.setPrice(Double.parseDouble(cellValue));//单价
                        }
                    } else if (j == 9) {
                        if (cellValue != ""){
                            entity.setCount(Long.parseLong(cellValue));//数量
                        }
                    } else if (j == 10) {
                        if (cellValue != ""){
                            entity.setTotal(Long.parseLong(cellValue));//总价
                        }
                    } else if (j == 11) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 12) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TOfficesupply t : list
        ) {
            failNum++;
            try {
                tOfficesupplyService.insertTOfficesupply(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
