package com.ruoyi.project.ehs.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.ehs.domain.TFirehose;
import com.ruoyi.project.ehs.service.ITFirehoseService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 消防水带箱Controller
 *
 * @author ruoyi
 * @date 2020-11-25
 */
@RestController
@RequestMapping("/ehs/firehose")
public class TFirehoseController extends BaseController
{
    @Autowired
    private ITFirehoseService tFirehoseService;

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    /**
     * 查询消防水带箱列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:firehose:list')")
    @GetMapping("/list")
    public TableDataInfo list(TFirehose tFirehose)
    {
        startPage();
        List<TFirehose> list = tFirehoseService.selectTFirehoseList(tFirehose);
        return getDataTable(list);
    }

    /**
     * 导出消防水带箱列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:firehose:export')")
    @Log(title = "消防水带箱", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TFirehose tFirehose)
    {
        List<TFirehose> list = tFirehoseService.selectTFirehoseList(tFirehose);
        ExcelUtil<TFirehose> util = new ExcelUtil<TFirehose>(TFirehose.class);
        return util.exportExcel(list, "firehose");
    }

    /**
     * 获取消防水带箱详细信息
     */
    @PreAuthorize("@ss.hasPermi('ehs:firehose:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tFirehoseService.selectTFirehoseById(id));
    }

    /**
     * 新增消防水带箱
     */
    @PreAuthorize("@ss.hasPermi('ehs:firehose:add')")
    @Log(title = "消防水带箱", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TFirehose tFirehose)
    {
        tFirehose.setCreaterCode(getUserId().toString());
        return toAjax(tFirehoseService.insertTFirehose(tFirehose));
    }

    /**
     * 修改消防水带箱
     */
    @PreAuthorize("@ss.hasPermi('ehs:firehose:edit')")
    @Log(title = "消防水带箱", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TFirehose tFirehose)
    {
        tFirehose.setUpdaterCode(getUserId().toString());
        tFirehose.setUpdatedate(new Date());
        return toAjax(tFirehoseService.updateTFirehose(tFirehose));
    }

    /**
     * 批量修改消防水带箱
     */
    @PreAuthorize("@ss.hasPermi('ehs:firehose:edit')")
    @Log(title = "消防水带箱", businessType = BusinessType.OTHER)
    @RequestMapping("/batchEdit")
    public AjaxResult batchEdit(@RequestBody TFirehose dto)
    {
        Long[] ids = dto.getIds();
        for (int i = 0; i < ids.length; i++) {
            TFirehose t = new TFirehose();
            t.setId(ids[i]);
            t.setRemarks(dto.getRemarks());
            t.setInspectstatus(dto.getInspectstatus());
            t.setUpdaterCode(getUserId().toString());
            t.setUpdatedate(new Date());
            tFirehoseService.updateTFirehose(t);
        }
        return AjaxResult.success();
    }

    /**
     * 删除消防水带箱
     */
    @PreAuthorize("@ss.hasPermi('ehs:firehose:remove')")
    @Log(title = "消防水带箱", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tFirehoseService.deleteTFirehoseByIds(ids));
    }

    /**
     * 批量导入消防水带箱
     */
    @PreAuthorize("@ss.hasPermi('ehs:firehose:add')")
    @Log(title = "消防水带箱", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TFirehose> list = new ArrayList<TFirehose>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        List<SysDictData> inspectstatus = iSysDictTypeService.selectDictDataByType("INSPECTSTATUS");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TFirehose entity = new TFirehose();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        entity.setNumb(cellValue);//编号
                    } else if (j == 2) {
                        entity.setName(cellValue);//名称
                    } else if (j == 3) {
                        entity.setArea(cellValue);//区域
                    } else if (j == 4) {
                        entity.setPosition(cellValue);//位置
                    } else if (j == 5) {
                        if (cellValue.length() > 3) {
                            entity.setInspectdate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//检查日期
                        }
                    } else if (j == 6) {
                        for (SysDictData p : inspectstatus) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setInspectstatus(p.getDictValue());//检查状况
                            }
                        }
                    } else if (j == 7) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 8) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TFirehose t : list
        ) {
            failNum++;
            try {
                tFirehoseService.insertTFirehose(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
