package com.ruoyi.project.ehs.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.ehs.domain.TWasteWater;
import com.ruoyi.project.ehs.service.ITWasteWaterService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 废水风险评估Controller
 *
 * @author ruoyi
 * @date 2020-12-08
 */
@RestController
@RequestMapping("/ehs/water")
public class TWasteWaterController extends BaseController
{
    @Autowired
    private ITWasteWaterService tWasteWaterService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    @Autowired
    private ISysDeptService iSysDeptService;

    /**
     * 查询废水风险评估列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:water:list')")
    @GetMapping("/list")
    public TableDataInfo list(TWasteWater tWasteWater)
    {
        startPage();
        List<TWasteWater> list = tWasteWaterService.selectTWasteWaterList(tWasteWater);
        return getDataTable(list);
    }

    /**
     * 导出废水风险评估列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:water:export')")
    @Log(title = "废水风险评估", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TWasteWater tWasteWater)
    {
        List<TWasteWater> list = tWasteWaterService.selectTWasteWaterList(tWasteWater);
        ExcelUtil<TWasteWater> util = new ExcelUtil<TWasteWater>(TWasteWater.class);
        return util.exportExcel(list, "water");
    }

    /**
     * 获取废水风险评估详细信息
     */
    @PreAuthorize("@ss.hasPermi('ehs:water:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tWasteWaterService.selectTWasteWaterById(id));
    }

    /**
     * 新增废水风险评估
     */
    @PreAuthorize("@ss.hasPermi('ehs:water:add')")
    @Log(title = "废水风险评估", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TWasteWater tWasteWater)
    {
        tWasteWater.setCreaterCode(getUserId().toString());
        return toAjax(tWasteWaterService.insertTWasteWater(tWasteWater));
    }

    /**
     * 修改废水风险评估
     */
    @PreAuthorize("@ss.hasPermi('ehs:water:edit')")
    @Log(title = "废水风险评估", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TWasteWater tWasteWater)
    {
        tWasteWater.setUpdaterCode(getUserId().toString());
        tWasteWater.setUpdatedate(new Date());
        return toAjax(tWasteWaterService.updateTWasteWater(tWasteWater));
    }

    /**
     * 删除废水风险评估
     */
    @PreAuthorize("@ss.hasPermi('ehs:water:remove')")
    @Log(title = "废水风险评估", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tWasteWaterService.deleteTWasteWaterByIds(ids));
    }

    /**
     * 批量导入废水风险评估
     */
    @PreAuthorize("@ss.hasPermi('ehs:water:add')")
    @Log(title = "废水风险评估", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TWasteWater> list = new ArrayList<TWasteWater>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TWasteWater entity = new TWasteWater();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        entity.setWaterName(cellValue);//污水名称
                    } else if (j == 2) {
                        entity.setFlow(cellValue);//流量
                    } else if (j == 3) {
                        entity.setOrigin(cellValue);//水源
                    } else if (j == 4) {
                        entity.setWwtp(cellValue);//污水处理厂
                    } else if (j == 5) {
                        entity.setDischargeTo(cellValue);//排放至
                    } else if (j == 6) {
                        entity.setSeverity(cellValue);//严重
                    } else if (j == 7) {
                        entity.setProbability(cellValue);//可能性
                    } else if (j == 8) {
                        entity.setRisk(cellValue);//风险
                    } else if (j == 9) {
                        entity.setExistsMeasures(cellValue);//现有防护措施
                    } else if (j == 10) {
                        entity.setResidualRisk(cellValue);//残余风险
                    } else if (j == 11) {
                        entity.setFurtherMeasures(cellValue);//进一步措施
                    } else if (j == 12) {
                        entity.setOwner(cellValue);//负责人
                    } else if (j == 13) {
                        if (cellValue.length() > 3) {
                            entity.setDeadline(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//截止日期
                        }
                    } else if (j == 14) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 15) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TWasteWater t : list
        ) {
            failNum++;
            try {
                tWasteWaterService.insertTWasteWater(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
