package com.ruoyi.project.patrol.emergency.controller;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.interceptor.annotation.RepeatSubmit;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.patrol.emergency.domain.TEmergencyCofferdam;
import com.ruoyi.project.patrol.emergency.service.ITEmergencyCofferdamService;
import com.ruoyi.project.patrol.patrol.domain.TPatrolIssues;
import com.ruoyi.project.patrol.patrol.domain.TPatrolPlan;
import com.ruoyi.project.patrol.patrol.service.ITPatrolIssuesService;
import com.ruoyi.project.patrol.patrol.service.ITPatrolPlanService;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 蒸汽裂解装置各区域围堰Controller
 *
 * @author admin
 * @date 2023-11-06
 */
@RestController
@RequestMapping("/emergency/cofferdam")
public class TEmergencyCofferdamController extends BaseController {
    @Autowired
    private ITEmergencyCofferdamService tEmergencyCofferdamService;

    @Autowired
    private ITPatrolPlanService tPatrolPlanService;

    @Autowired
    private ITPatrolIssuesService tPatrolIssuesService;

    @Autowired
    private ISysUserService sysUserService;

    /**
     * 查询蒸汽裂解装置各区域围堰列表
     */
    @PreAuthorize("@ss.hasPermi('emergency:cofferdam:list')")
    @GetMapping("/list")
    public TableDataInfo list(TEmergencyCofferdam tEmergencyCofferdam) {
        startPage();
        if (StringUtils.isEmpty(tEmergencyCofferdam.getIssuesFlag()))
            tEmergencyCofferdam.setIssuesFlag("0");
        List<TEmergencyCofferdam> list = tEmergencyCofferdamService.selectTEmergencyCofferdamList(tEmergencyCofferdam);
        return getDataTable(list);
    }

    /**
     * 导出蒸汽裂解装置各区域围堰列表
     */
    
    @Log(title = "蒸汽裂解装置各区域围堰", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public AjaxResult export(TEmergencyCofferdam tEmergencyCofferdam) {
        try {
            tEmergencyCofferdam.setIssuesFlag("0");
            List<TEmergencyCofferdam> list = tEmergencyCofferdamService.selectTEmergencyCofferdamList(tEmergencyCofferdam);
            XSSFWorkbook workbook = new XSSFWorkbook();
            // 创建工作簿
            XSSFSheet sheet = workbook.createSheet();
            // 设置列宽
            sheet.setColumnWidth(0, 1200);
            sheet.setColumnWidth(1, 3500);
            sheet.setColumnWidth(2, 3500);
            sheet.setColumnWidth(3, 4500);
            sheet.setColumnWidth(4, 4500);
            sheet.setColumnWidth(5, 5500);
            sheet.setColumnWidth(6, 4500);
            //第一行
            XSSFRow row1 = sheet.createRow(0);
            row1.createCell(0).setCellValue("CBP-6.3.0-SCFSH-020.20  Rev:04  2021/10/01");
            ExcelUtils.createVoidCell(1, 6, row1);
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 6));//合并
            //第二行
            XSSFRow row2 = sheet.createRow(1);
            row2.createCell(0).setCellValue("蒸汽裂解装置各区域围堰检查记录");
            ExcelUtils.createVoidCell(1, 6, row2);
            sheet.addMergedRegion(new CellRangeAddress(1, 1, 0, 6));
            //第三行
            XSSFRow row3 = sheet.createRow(2);
            row3.createCell(0).setCellValue("年度：" + tEmergencyCofferdam.getYear());
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 0, 1));
            ExcelUtils.createVoidCell(1, 6, row3);
            row3.createCell(2).setCellValue("检查频率：1次/3月");
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 2, 6));
            //第四行
            XSSFRow row4 = sheet.createRow(3);
            row4.createCell(0).setCellValue("序号");
            row4.createCell(1).setCellValue("位号");
            row4.createCell(2).setCellValue("检查内容");
            ExcelUtils.createVoidCell(4, 6, row4);
            sheet.addMergedRegion(new CellRangeAddress(3, 3, 2, 6));
            //第五行
            XSSFRow row5 = sheet.createRow(4);
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 0, 0));
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 1, 1));
            ExcelUtils.createVoidCell(0, 2, row5);
            row5.createCell(2).setCellValue("地面完整度（裂缝、孔洞等）");
            row5.createCell(3).setCellValue("墙体完整度（裂缝、孔洞等）");
            row5.createCell(4).setCellValue("地面排水及积水情况");
            row5.createCell(5).setCellValue("围堰内卫生");
            row5.createCell(6).setCellValue("检查人");
            Long planId = null;
            //遍历数组赋值
            for (int i = 0; i < list.size(); i++) {
                TEmergencyCofferdam cofferdam = list.get(i);
                planId = cofferdam.getPlanId();
                XSSFRow row = sheet.createRow(5 + i);
                row.createCell(0).setCellValue(i + 1);
                row.createCell(1).setCellValue(cofferdam.getDevNo());
                row.createCell(2).setCellValue("1".equals(cofferdam.getGroundIntegrity()) ? "√" : cofferdam.getGroundIntegrity());
                row.createCell(3).setCellValue("1".equals(cofferdam.getWallIntegrity()) ? "√" : cofferdam.getWallIntegrity());
                row.createCell(4).setCellValue("1".equals(cofferdam.getWaterCondition()) ? "√" : cofferdam.getWaterCondition());
                row.createCell(5).setCellValue("1".equals(cofferdam.getCofferdamHygiene()) ? "√" : cofferdam.getCofferdamHygiene());
                XSSFCell cell10 = row.createCell(6);
                if (StringUtils.isNotEmpty(cofferdam.getUpdaterCode())) {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(cofferdam.getUpdaterCode()));
                    cell10.setCellValue(sysUser == null ? "" : sysUser.getNickName());
                }
            }
            String confirmed = "";
            Date confirmedDate = null;
            TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(planId);
            if (Objects.nonNull(tPatrolPlan) && Objects.nonNull(tPatrolPlan.getConfirmed())) {
                SysUser sysUser = sysUserService.selectUserById(Long.valueOf(tPatrolPlan.getConfirmed()));            if (sysUser!=null) {
                confirmed = sysUser.getSignUrl();
                confirmedDate = tPatrolPlan.getConfirmDate();
            }
            }
            // 循环后下一行
            int nextRowNum = 5 + list.size();
            XSSFRow row28 = sheet.createRow(nextRowNum);
            row28.setHeightInPoints(30);
            XSSFCell row28Cell1 = row28.createCell(0);
            row28Cell1.setCellValue("签名");
            ExcelUtils.createVoidCell(1, 2, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
            if (StringUtils.isNotEmpty(confirmed))
                ExcelUtils.insertPicture(workbook, sheet, confirmed, nextRowNum, 3, 1, 1);
            ExcelUtils.createVoidCell(2, 6, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 6));
            // 下一行
            nextRowNum++;
            XSSFRow row29 = sheet.createRow(nextRowNum);
            row29.setHeightInPoints(30);
            XSSFCell row29Cell1 = row29.createCell(0);
            row29Cell1.setCellValue("日期");
            ExcelUtils.createVoidCell(1, 1, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
            row29.createCell(2).setCellValue(confirmedDate != null ? new SimpleDateFormat("yyyy-MM-dd").format(confirmedDate) : "");
            ExcelUtils.createVoidCell(3, 6, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 6));
            // 设置单元格样式
            // 文字居中，自动换行，四周边框
            CellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            style.setVerticalAlignment(VerticalAlignment.CENTER);
            style.setWrapText(true);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);

            // 设置垂直居中，四周边框
            CellStyle style2 = workbook.createCellStyle();
            style2.setVerticalAlignment(VerticalAlignment.CENTER);
            style2.setBorderTop(BorderStyle.THIN);
            style2.setBorderBottom(BorderStyle.THIN);
            style2.setBorderLeft(BorderStyle.THIN);
            style2.setBorderRight(BorderStyle.THIN);
            for (Row row : sheet) {
                row.setHeightInPoints(30);
                if (row.getRowNum() == 0) {
                    continue;
                }
                if (row.getRowNum() == 4) {

                    row.setHeightInPoints(70);
                }
                if (row.getRowNum() == 2) {
                    for (Cell cell : row) {
                        if (cell.getColumnIndex() == 0 || cell.getColumnIndex() == 1)
                            cell.setCellStyle(style2);
                        else
                            cell.setCellStyle(style);
                    }
                } else {
                    for (Cell cell : row) {
                        cell.setCellStyle(style);
                    }
                }
            }
            nextRowNum++;
            XSSFRow row30 = sheet.createRow(nextRowNum);
            row30.createCell(0).setCellValue("说明：");
            ExcelUtils.createVoidCell(1, 6, row30);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 6));

            nextRowNum++;
            XSSFRow row31 = sheet.createRow(nextRowNum);
            row31.createCell(0).setCellValue("1.检查正常打“√”。");
            ExcelUtils.createVoidCell(1, 6, row31);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 6));

            nextRowNum++;
            XSSFRow row32 = sheet.createRow(nextRowNum);
            row32.createCell(0).setCellValue("2.检查有不正常情况打“X”，并记录存在问题，处理结果及完成时间。");
            ExcelUtils.createVoidCell(1, 6, row32);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 6));

            nextRowNum++;
            XSSFRow row33 = sheet.createRow(nextRowNum);
            row33.createCell(0).setCellValue("3. 检查频率为每3个月检查维护一次。");
            ExcelUtils.createVoidCell(1, 6, row33);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 6));

            //返回
            String fileName = ExcelUtil.encodingFilename("main");
            OutputStream os = Files.newOutputStream(Paths.get(ExcelUtil.getAbsoluteFile(fileName)));
            workbook.write(os);
            workbook.close();
            return AjaxResult.success(fileName);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取蒸汽裂解装置各区域围堰详细信息
     */
    @PreAuthorize("@ss.hasPermi('emergency:cofferdam:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tEmergencyCofferdamService.selectTEmergencyCofferdamById(id));
    }

    /**
     * 获取蒸汽裂解装置各区域围堰详细信息
     */
    @PreAuthorize("@ss.hasPermi('emergency:cofferdam:query')")
    @GetMapping(value = "/info")
    public AjaxResult getInfo(TEmergencyCofferdam tEmergencyCofferdam) {
        if (StringUtils.isEmpty(tEmergencyCofferdam.getDevNo()) || StringUtils.isNull(tEmergencyCofferdam.getPlanId())) {
            return AjaxResult.error("位号或计划id不能为空！");
        }
        TEmergencyCofferdam item = new TEmergencyCofferdam();
        item.setPlanId(tEmergencyCofferdam.getPlanId());
        item.setPrecision(tEmergencyCofferdam.getDevNo());
        item.setIssuesFlag("0");
        List<TEmergencyCofferdam> list = tEmergencyCofferdamService.selectTEmergencyCofferdamList(item);
        if (CollectionUtils.isNotEmpty(list))
            return AjaxResult.success(list.get(0));
        return AjaxResult.error("此设备不在当前巡检计划中!");
    }


    /**
     * 新增蒸汽裂解装置各区域围堰
     */
    @PreAuthorize("@ss.hasPermi('emergency:cofferdam:add')")
    @Log(title = "蒸汽裂解装置各区域围堰", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TEmergencyCofferdam tEmergencyCofferdam) {
        return toAjax(tEmergencyCofferdamService.insertTEmergencyCofferdam(tEmergencyCofferdam));
    }

    /**
     * 修改蒸汽裂解装置各区域围堰
     */
    @PreAuthorize("@ss.hasPermi('emergency:cofferdam:edit')")
    @Log(title = "蒸汽裂解装置各区域围堰", businessType = BusinessType.UPDATE)
    @PutMapping
    @RepeatSubmit
    public AjaxResult edit(@RequestBody TEmergencyCofferdam tEmergencyCofferdam) {
        tEmergencyCofferdam.setUpdatedate(new Date());
        tEmergencyCofferdam.setUpdaterCode(getUserId().toString());
        tEmergencyCofferdam.setCheckStatus("1");
        tEmergencyCofferdam.setCheckDate(new Date());
        TEmergencyCofferdam item = tEmergencyCofferdamService.selectTEmergencyCofferdamById(tEmergencyCofferdam.getId());
        TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(tEmergencyCofferdam.getPlanId());
        if (tPatrolPlan.getStatus() == 1) {
            return AjaxResult.error("当前选择的计划已确认！");
        }
        if (!"1".equals(item.getCheckStatus())) {
            tPatrolPlan.setUncheckedNum(tPatrolPlan.getUncheckedNum() - 1);
            tPatrolPlan.setCheckedNum(tPatrolPlan.getCheckedNum() + 1);
            tPatrolPlanService.updateTPatrolPlan(tPatrolPlan);
        }
        if ((StringUtils.isNotEmpty(tEmergencyCofferdam.getCofferdamHygiene()) && !"1".equals(tEmergencyCofferdam.getCofferdamHygiene())) ||
                (StringUtils.isNotEmpty(tEmergencyCofferdam.getGroundIntegrity()) && !"1".equals(tEmergencyCofferdam.getGroundIntegrity())) ||
                (StringUtils.isNotEmpty(tEmergencyCofferdam.getWallIntegrity()) && !"1".equals(tEmergencyCofferdam.getWallIntegrity())) ||
                (StringUtils.isNotEmpty(tEmergencyCofferdam.getWaterCondition()) && !"1".equals(tEmergencyCofferdam.getWaterCondition()))) {
            tEmergencyCofferdam.setIssuesStatus("1");
            tEmergencyCofferdam.setCheckStatus("0");
            tEmergencyCofferdamService.updateTEmergencyCofferdam(tEmergencyCofferdam);
            tEmergencyCofferdam.setIssuesFlag("1");
            tEmergencyCofferdam.setCheckStatus("1");
            tEmergencyCofferdam.setCreatedate(new Date());
            tEmergencyCofferdam.setCreaterCode(getUserId().toString());
            tEmergencyCofferdam.setDevNo(item.getDevNo());
            tEmergencyCofferdamService.insertTEmergencyCofferdam(tEmergencyCofferdam);
            TPatrolIssues tPatrolIssues = new TPatrolIssues();
            tPatrolIssues.setPatrolType("17");
            tPatrolIssues.setDevDescribe(tEmergencyCofferdam.getDevNo());
            tPatrolIssues.setLinkId(tEmergencyCofferdam.getId());
            tPatrolIssues.setStatus(0L);
            tPatrolIssuesService.insertTPatrolIssues(tPatrolIssues);
        } else {
            if ("1".equals(item.getIssuesStatus())) {
                TPatrolIssues tPatrolIssues = new TPatrolIssues();
                tPatrolIssues.setPatrolType("17");
                tPatrolIssues.setDevDescribe(item.getDevNo());
                tPatrolIssues.setCreatedate(new Date());
                for (TPatrolIssues patrolIssues : tPatrolIssuesService.selectTPatrolIssuesList(tPatrolIssues)) {
                    patrolIssues.setStatus(1L);
                    tPatrolIssuesService.updateTPatrolIssues(patrolIssues);
                }
            }
            tEmergencyCofferdam.setIssuesStatus("0");
            tEmergencyCofferdamService.updateTEmergencyCofferdam(tEmergencyCofferdam);
        }
        return AjaxResult.success();
    }

    /**
     * 删除蒸汽裂解装置各区域围堰
     */
    @PreAuthorize("@ss.hasPermi('emergency:cofferdam:remove')")
    @Log(title = "蒸汽裂解装置各区域围堰", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tEmergencyCofferdamService.deleteTEmergencyCofferdamByIds(ids));
    }
}
