package com.ruoyi.project.patrol.fire.controller;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.interceptor.annotation.RepeatSubmit;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.patrol.dev.domain.TDevAlarm;
import com.ruoyi.project.patrol.dev.service.ITDevAlarmService;
import com.ruoyi.project.patrol.fire.domain.TFireAlarm;
import com.ruoyi.project.patrol.fire.service.ITFireAlarmService;
import com.ruoyi.project.patrol.patrol.domain.TPatrolIssues;
import com.ruoyi.project.patrol.patrol.domain.TPatrolPlan;
import com.ruoyi.project.patrol.patrol.service.ITPatrolIssuesService;
import com.ruoyi.project.patrol.patrol.service.ITPatrolPlanService;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

/**
 * 现场可燃气体检测器/火灾报警系统Controller
 *
 * @author admin
 * @date 2023-11-01
 */
@RestController
@RequestMapping("/fire/alarm")
public class TFireAlarmController extends BaseController {
    @Autowired
    private ITFireAlarmService tFireAlarmService;

    @Autowired
    private ITDevAlarmService tDevAlarmService;

    @Autowired
    private ITPatrolPlanService tPatrolPlanService;

    @Autowired
    private ITPatrolIssuesService tPatrolIssuesService;

    @Autowired
    private ISysUserService sysUserService;

    /**
     * 查询现场可燃气体检测器/火灾报警系统列表
     */
    @PreAuthorize("@ss.hasPermi('fire:alarm:list')")
    @GetMapping("/list")
    public TableDataInfo list(TFireAlarm tFireAlarm) {
        startPage();
        if (StringUtils.isEmpty(tFireAlarm.getIssuesFlag()))
            tFireAlarm.setIssuesFlag("0");
        List<TFireAlarm> list = tFireAlarmService.selectTFireAlarmList(tFireAlarm);
        return getDataTable(list);
    }

    /**
     * 导出现场可燃气体检测器/火灾报警系统列表
     */
    
    @Log(title = "现场可燃气体检测器/火灾报警系统", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public AjaxResult export(TFireAlarm tFireAlarm) {
        try {
            tFireAlarm.setIssuesFlag("0");
            List<TFireAlarm> list = tFireAlarmService.selectTFireAlarmList(tFireAlarm);
            XSSFWorkbook workbook = new XSSFWorkbook();
            // 创建工作簿
            XSSFSheet sheet = workbook.createSheet();
            // 设置列宽
            sheet.setColumnWidth(0, 1200);
            sheet.setColumnWidth(1, 3500);
            sheet.setColumnWidth(2, 3500);
            sheet.setColumnWidth(3, 4500);
            sheet.setColumnWidth(4, 4500);
            sheet.setColumnWidth(5, 5500);
            sheet.setColumnWidth(6, 4500);
            sheet.setColumnWidth(7, 4500);
            //第一行
            XSSFRow row1 = sheet.createRow(0);
            row1.createCell(0).setCellValue("CBP-6.3.0-SCFSH-020.10  Rev:04  2021/10/01");
            ExcelUtils.createVoidCell(1, 7, row1);
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 7));//合并
            //第二行
            XSSFRow row2 = sheet.createRow(1);
            row2.createCell(0).setCellValue("现场可燃气体检测器/火灾报警系统检查记录");
            ExcelUtils.createVoidCell(1, 7, row2);
            sheet.addMergedRegion(new CellRangeAddress(1, 1, 0, 7));
            //第三行
            XSSFRow row3 = sheet.createRow(2);
            row3.createCell(0).setCellValue("年度：" + tFireAlarm.getYear());
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 0, 1));
            ExcelUtils.createVoidCell(1, 7, row3);
            row3.createCell(2).setCellValue("检查频率：1次/3月");
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 2, 7));
            //第四行
            XSSFRow row4 = sheet.createRow(3);
            row4.createCell(0).setCellValue("序号");
            row4.createCell(1).setCellValue("位号");
            row4.createCell(2).setCellValue("检查内容");
            ExcelUtils.createVoidCell(4, 7, row4);
            sheet.addMergedRegion(new CellRangeAddress(3, 3, 2, 7));
            //第五行
            XSSFRow row5 = sheet.createRow(4);
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 0, 0));
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 1, 1));
            ExcelUtils.createVoidCell(0, 2, row5);
            row5.createCell(2).setCellValue("现场检测器外观及防雨罩");
            row5.createCell(3).setCellValue("检测器位号及分类标识");
            row5.createCell(4).setCellValue("检测器分布点位图及版本号");
            row5.createCell(5).setCellValue("控制室显示面板报警类型、指示灯内容与现场相符");
            row5.createCell(6).setCellValue("现场火灾报警系统按钮玻璃");
            row5.createCell(7).setCellValue("检查人");
            Long planId = null;
            //遍历数组赋值
            for (int i = 0; i < list.size(); i++) {
                TFireAlarm hoseBox = list.get(i);
                planId = hoseBox.getPlanId();
                XSSFRow row = sheet.createRow(5 + i);
                row.createCell(0).setCellValue(i + 1);
                row.createCell(1).setCellValue(hoseBox.getDevNo());
                TDevAlarm tDevAlarm = new TDevAlarm();
                tDevAlarm.setDevNo(hoseBox.getDevNo());
                TDevAlarm item = tDevAlarmService.selectTDevAlarmList(tDevAlarm).get(0);
                if (StringUtils.isNotEmpty(item.getIgnoreColumn()) && item.getIgnoreColumn().contains("detectorAppearance")) {
                    row.createCell(2).setCellValue("/");
                } else
                    row.createCell(2).setCellValue("1".equals(hoseBox.getDetectorAppearance()) ? "√" : hoseBox.getDetectorAppearance());
                if (StringUtils.isNotEmpty(item.getIgnoreColumn()) && item.getIgnoreColumn().contains("digitIdentification")) {
                    row.createCell(3).setCellValue("/");
                } else
                    row.createCell(3).setCellValue("1".equals(hoseBox.getDigitIdentification()) ? "√" : hoseBox.getDigitIdentification());
                if (StringUtils.isNotEmpty(item.getIgnoreColumn()) && item.getIgnoreColumn().contains("versionNumber")) {
                    row.createCell(4).setCellValue("/");
                } else
                    row.createCell(4).setCellValue("1".equals(hoseBox.getVersionNumber()) ? "√" : hoseBox.getVersionNumber());
                if (StringUtils.isNotEmpty(item.getIgnoreColumn()) && item.getIgnoreColumn().contains("accordWith")) {
                    row.createCell(5).setCellValue("/");
                } else
                    row.createCell(5).setCellValue("1".equals(hoseBox.getAccordWith()) ? "√" : hoseBox.getAccordWith());
                if (StringUtils.isNotEmpty(item.getIgnoreColumn()) && item.getIgnoreColumn().contains("buttonGlass")) {
                    row.createCell(6).setCellValue("/");
                } else
                    row.createCell(6).setCellValue("1".equals(hoseBox.getButtonGlass()) ? "√" : hoseBox.getButtonGlass());
                XSSFCell cell10 = row.createCell(7);
                if (StringUtils.isNotEmpty(hoseBox.getUpdaterCode())) {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(hoseBox.getUpdaterCode()));
                    cell10.setCellValue(sysUser == null ? "" : sysUser.getNickName());
                }
            }
            String confirmed = "";
            Date confirmedDate = null;
            TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(planId);
            if (Objects.nonNull(tPatrolPlan) && Objects.nonNull(tPatrolPlan.getConfirmed())) {
                SysUser sysUser = sysUserService.selectUserById(Long.valueOf(tPatrolPlan.getConfirmed()));            if (sysUser!=null) {
                confirmed = sysUser.getSignUrl();
                confirmedDate = tPatrolPlan.getConfirmDate();
            }
            }
            // 循环后下一行
            int nextRowNum = 5 + list.size();
            XSSFRow row28 = sheet.createRow(nextRowNum);
            row28.setHeightInPoints(30);
            XSSFCell row28Cell1 = row28.createCell(0);
            row28Cell1.setCellValue("签名");
            ExcelUtils.createVoidCell(1, 2, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
            if (StringUtils.isNotEmpty(confirmed))
                ExcelUtils.insertPicture(workbook, sheet, confirmed, nextRowNum, 5, 1, 1);
            ExcelUtils.createVoidCell(2, 7, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 7));
            // 下一行
            nextRowNum++;
            XSSFRow row29 = sheet.createRow(nextRowNum);
            row29.setHeightInPoints(30);
            XSSFCell row29Cell1 = row29.createCell(0);
            row29Cell1.setCellValue("日期");
            ExcelUtils.createVoidCell(1, 1, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
            row29.createCell(2).setCellValue(confirmedDate != null ? new SimpleDateFormat("yyyy-MM-dd").format(confirmedDate) : "");
            ExcelUtils.createVoidCell(3, 7, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 7));
            // 设置单元格样式
            // 文字居中，自动换行，四周边框
            CellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            style.setVerticalAlignment(VerticalAlignment.CENTER);
            style.setWrapText(true);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);

            // 设置垂直居中，四周边框
            CellStyle style2 = workbook.createCellStyle();
            style2.setVerticalAlignment(VerticalAlignment.CENTER);
            style2.setBorderTop(BorderStyle.THIN);
            style2.setBorderBottom(BorderStyle.THIN);
            style2.setBorderLeft(BorderStyle.THIN);
            style2.setBorderRight(BorderStyle.THIN);
            for (Row row : sheet) {
                row.setHeightInPoints(30);
                if (row.getRowNum() == 0) {
                    continue;
                }
                if (row.getRowNum() == 2) {
                    for (Cell cell : row) {
                        if (cell.getColumnIndex() == 0 || cell.getColumnIndex() == 1)
                            cell.setCellStyle(style2);
                        else
                            cell.setCellStyle(style);
                    }
                } else {
                    for (Cell cell : row) {
                        cell.setCellStyle(style);
                    }
                }
            }
            nextRowNum++;
            XSSFRow row30 = sheet.createRow(nextRowNum);
            row30.createCell(0).setCellValue("说明：");
            ExcelUtils.createVoidCell(1, 7, row30);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 7));

            nextRowNum++;
            XSSFRow row31 = sheet.createRow(nextRowNum);
            row31.createCell(0).setCellValue("1.检查正常打“√”。");
            ExcelUtils.createVoidCell(1, 7, row31);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 7));

            nextRowNum++;
            XSSFRow row32 = sheet.createRow(nextRowNum);
            row32.createCell(0).setCellValue("2.检查有不正常情况打“X”，并记录存在问题，处理结果及完成时间。");
            ExcelUtils.createVoidCell(1, 7, row32);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 7));

            nextRowNum++;
            XSSFRow row33 = sheet.createRow(nextRowNum);
            row33.createCell(0).setCellValue("3. 检查频率为每三个月检查维护一次。");
            ExcelUtils.createVoidCell(1, 7, row33);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 7));

            //返回
            String fileName = "PSSR_13_消气防设施_现场可燃气体检测器、火灾报警系统检查记录.xlsx";
            OutputStream os = Files.newOutputStream(Paths.get(ExcelUtil.getAbsoluteFile(fileName)));
            workbook.write(os);
            workbook.close();
            return AjaxResult.success(fileName);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取现场可燃气体检测器/火灾报警系统详细信息
     */
    @PreAuthorize("@ss.hasPermi('fire:alarm:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TFireAlarm alarm = tFireAlarmService.selectTFireAlarmById(id);
        if (alarm!=null) {
            TDevAlarm tDevAlarm = new TDevAlarm();
            tDevAlarm.setPrecision(alarm.getDevNo());
            alarm.setIgnoreColumn(tDevAlarmService.selectTDevAlarmList(tDevAlarm).get(0).getIgnoreColumn());
        }
        return AjaxResult.success(alarm);
    }

    /**
     * 获取现场可燃气体检测器/火灾报警系统详细信息
     */
    @PreAuthorize("@ss.hasPermi('fire:alarm:query')")
    @GetMapping(value = "/info")
    public AjaxResult getInfo(TFireAlarm tFireAlarm) {
        if (StringUtils.isEmpty(tFireAlarm.getDevNo()) || StringUtils.isNull(tFireAlarm.getPlanId())) {
            return AjaxResult.error("位号或计划id不能为空！");
        }
        TFireAlarm item = new TFireAlarm();
        item.setPlanId(tFireAlarm.getPlanId());
        item.setPrecision(tFireAlarm.getDevNo());
        item.setIssuesFlag("0");
        List<TFireAlarm> list = tFireAlarmService.selectTFireAlarmList(item);
        if (CollectionUtils.isNotEmpty(list)) {
            TFireAlarm alarm = list.get(0);
            TDevAlarm tDevAlarm = new TDevAlarm();
            tDevAlarm.setPrecision(alarm.getDevNo());
            alarm.setIgnoreColumn(tDevAlarmService.selectTDevAlarmList(tDevAlarm).get(0).getIgnoreColumn());
            return AjaxResult.success(alarm);
        }
        return AjaxResult.error("此设备不在当前巡检计划中!");
    }

    /**
     * 新增现场可燃气体检测器/火灾报警系统
     */
    @PreAuthorize("@ss.hasPermi('fire:alarm:add')")
    @Log(title = "现场可燃气体检测器/火灾报警系统", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TFireAlarm tFireAlarm) {
        return toAjax(tFireAlarmService.insertTFireAlarm(tFireAlarm));
    }

    /**
     * 修改现场可燃气体检测器/火灾报警系统
     */
    @PreAuthorize("@ss.hasPermi('fire:alarm:edit')")
    @Log(title = "现场可燃气体检测器/火灾报警系统", businessType = BusinessType.UPDATE)
    @PutMapping
    @RepeatSubmit
    public AjaxResult edit(@RequestBody TFireAlarm tFireAlarm) {
        tFireAlarm.setUpdatedate(new Date());
        tFireAlarm.setUpdaterCode(getUserId().toString());
        tFireAlarm.setCheckStatus("1");
        tFireAlarm.setCheckDate(new Date());
        TFireAlarm item = tFireAlarmService.selectTFireAlarmById(tFireAlarm.getId());
        TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(tFireAlarm.getPlanId());
        if (tPatrolPlan.getStatus() == 1) {
            return AjaxResult.error("当前选择的计划已确认！");
        }
        if (!"1".equals(item.getCheckStatus())) {
            tPatrolPlan.setUncheckedNum(tPatrolPlan.getUncheckedNum() - 1);
            tPatrolPlan.setCheckedNum(tPatrolPlan.getCheckedNum() + 1);
            tPatrolPlanService.updateTPatrolPlan(tPatrolPlan);
        }
        TDevAlarm tDevAlarm = new TDevAlarm();
        tDevAlarm.setDevNo(item.getDevNo());
        TDevAlarm alarm = tDevAlarmService.selectTDevAlarmList(tDevAlarm).get(0);
        if (StringUtils.isNotEmpty(alarm.getIgnoreColumn()) && alarm.getIgnoreColumn().contains("accordWith")) {
            tFireAlarm.setAccordWith("/");
        }
        if (StringUtils.isNotEmpty(alarm.getIgnoreColumn()) && alarm.getIgnoreColumn().contains("buttonGlass")) {
            tFireAlarm.setButtonGlass("/");
        }
        if (StringUtils.isNotEmpty(alarm.getIgnoreColumn()) && alarm.getIgnoreColumn().contains("detectorAppearance")) {
            tFireAlarm.setDetectorAppearance("/");
        }
        if (StringUtils.isNotEmpty(alarm.getIgnoreColumn()) && alarm.getIgnoreColumn().contains("digitIdentification")) {
            tFireAlarm.setDigitIdentification("/");
        }
        if (StringUtils.isNotEmpty(alarm.getIgnoreColumn()) && alarm.getIgnoreColumn().contains("versionNumber")) {
            tFireAlarm.setVersionNumber("/");
        }
        if (StringUtils.isNotEquals(tFireAlarm.getAccordWith(), "1", "/") || StringUtils.isNotEquals(tFireAlarm.getButtonGlass(), "1", "/") ||
                StringUtils.isNotEquals(tFireAlarm.getDetectorAppearance(), "1", "/") || StringUtils.isNotEquals(tFireAlarm.getDigitIdentification(), "1", "/") ||
                StringUtils.isNotEquals(tFireAlarm.getVersionNumber(), "1", "/")) {
            tFireAlarm.setIssuesStatus("1");
            tFireAlarm.setCheckStatus("0");
            tFireAlarmService.updateTFireAlarm(tFireAlarm);
            tFireAlarm.setIssuesFlag("1");
            tFireAlarm.setCheckStatus("1");
            tFireAlarm.setCreatedate(new Date());
            tFireAlarm.setCreaterCode(getUserId().toString());
            tFireAlarm.setDevNo(item.getDevNo());
            tFireAlarmService.insertTFireAlarm(tFireAlarm);
            TPatrolIssues tPatrolIssues = new TPatrolIssues();
            tPatrolIssues.setPatrolType("8");
            tPatrolIssues.setDevDescribe(tFireAlarm.getDevNo());
            tPatrolIssues.setLinkId(tFireAlarm.getId());
            tPatrolIssues.setStatus(0L);
            tPatrolIssuesService.insertTPatrolIssues(tPatrolIssues);
        } else {
            if ("1".equals(item.getIssuesStatus())) {
                TPatrolIssues tPatrolIssues = new TPatrolIssues();
                tPatrolIssues.setPatrolType("8");
                tPatrolIssues.setDevDescribe(item.getDevNo());
                tPatrolIssues.setCreatedate(new Date());
                for (TPatrolIssues patrolIssues : tPatrolIssuesService.selectTPatrolIssuesList(tPatrolIssues)) {
                    patrolIssues.setStatus(1L);
                    tPatrolIssuesService.updateTPatrolIssues(patrolIssues);
                }
            }
            tFireAlarm.setIssuesStatus("0");
            tFireAlarmService.updateTFireAlarm(tFireAlarm);
        }
        return AjaxResult.success();
    }

    /**
     * 删除现场可燃气体检测器/火灾报警系统
     */
    @PreAuthorize("@ss.hasPermi('fire:alarm:remove')")
    @Log(title = "现场可燃气体检测器/火灾报警系统", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tFireAlarmService.deleteTFireAlarmByIds(ids));
    }
}
