package com.ruoyi.project.patrol.fire.controller;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.interceptor.annotation.RepeatSubmit;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.patrol.fire.domain.TFireDoor;
import com.ruoyi.project.patrol.fire.service.ITFireDoorService;
import com.ruoyi.project.patrol.patrol.domain.TPatrolIssues;
import com.ruoyi.project.patrol.patrol.domain.TPatrolPlan;
import com.ruoyi.project.patrol.patrol.service.ITPatrolIssuesService;
import com.ruoyi.project.patrol.patrol.service.ITPatrolPlanService;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

/**
 * 常闭式防爆门/防火门Controller
 *
 * @author admin
 * @date 2023-11-07
 */
@RestController
@RequestMapping("/fire/door")
public class TFireDoorController extends BaseController {
    @Autowired
    private ITFireDoorService tFireDoorService;

    @Autowired
    private ITPatrolPlanService tPatrolPlanService;

    @Autowired
    private ITPatrolIssuesService tPatrolIssuesService;

    @Autowired
    private ISysUserService sysUserService;

    /**
     * 查询常闭式防爆门/防火门列表
     */
    @PreAuthorize("@ss.hasPermi('fire:door:list')")
    @GetMapping("/list")
    public TableDataInfo list(TFireDoor tFireDoor) {
        startPage();
        if (StringUtils.isEmpty(tFireDoor.getIssuesFlag()))
            tFireDoor.setIssuesFlag("0");
        List<TFireDoor> list = tFireDoorService.selectTFireDoorList(tFireDoor);
        return getDataTable(list);
    }

    /**
     * 导出常闭式防爆门/防火门列表
     */
    
    @Log(title = "常闭式防爆门/防火门", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public AjaxResult export(TFireDoor tFireDoor) {
        try {
            tFireDoor.setIssuesFlag("0");
            XSSFWorkbook workbook = new XSSFWorkbook();
            tFireDoor.setDevType("D301");
            generateSheet(workbook, tFireDoor, "6");
            tFireDoor.setDevType("D201");
            generateSheet(workbook, tFireDoor, "7");
            // 设置单元格样式
            // 文字居中，自动换行，四周边框
            CellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            style.setVerticalAlignment(VerticalAlignment.CENTER);
            style.setWrapText(true);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);

            // 设置垂直居中，四周边框
            CellStyle style2 = workbook.createCellStyle();
            style2.setVerticalAlignment(VerticalAlignment.CENTER);
            style2.setBorderTop(BorderStyle.THIN);
            style2.setBorderBottom(BorderStyle.THIN);
            style2.setBorderLeft(BorderStyle.THIN);
            style2.setBorderRight(BorderStyle.THIN);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                XSSFSheet sheet = workbook.getSheetAt(i);
                for (Row row : sheet) {
                    row.setHeightInPoints(30);
                    if (row.getRowNum() == 0) {
                        continue;
                    }
                    if (row.getRowNum() == 2) {
                        for (Cell cell : row) {
                            if (cell.getColumnIndex() == 0 || cell.getColumnIndex() == 1)
                                cell.setCellStyle(style2);
                            else
                                cell.setCellStyle(style);
                        }
                    } else {
                        for (Cell cell : row) {
                            cell.setCellStyle(style);
                        }
                    }
                }
                int nextRowNum = sheet.getLastRowNum();
                nextRowNum++;
                XSSFRow row30 = sheet.createRow(nextRowNum);
                row30.createCell(0).setCellValue("说明：");
                ExcelUtils.createVoidCell(1, 8, row30);
                sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 8));

                nextRowNum++;
                XSSFRow row31 = sheet.createRow(nextRowNum);
                row31.createCell(0).setCellValue("1.检查正常打“√”。");
                ExcelUtils.createVoidCell(1, 8, row31);
                sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 8));

                nextRowNum++;
                XSSFRow row32 = sheet.createRow(nextRowNum);
                row32.createCell(0).setCellValue("2.检查有不正常情况打“X”，并记录存在问题，处理结果及完成时间。");
                ExcelUtils.createVoidCell(1, 8, row32);
                sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 8));

                nextRowNum++;
                XSSFRow row33 = sheet.createRow(nextRowNum);
                row33.createCell(0).setCellValue("3. 检查频率为每一个月检查维护一次。");
                ExcelUtils.createVoidCell(1, 8, row33);
                sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 8));
            }
            //返回
            String fileName = "PSSR_13_消气防设施_常闭式防爆门、防火门检查记录.xlsx";
            OutputStream os = Files.newOutputStream(Paths.get(ExcelUtil.getAbsoluteFile(fileName)));
            workbook.write(os);
            workbook.close();
            return AjaxResult.success(fileName);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private void generateSheet(XSSFWorkbook workbook, TFireDoor tFireDoor, String val) throws IOException {
        List<TFireDoor> list = tFireDoorService.selectTFireDoorList(tFireDoor);
        // 创建工作簿
        XSSFSheet sheet = workbook.createSheet(tFireDoor.getDevType()+"常闭式防爆门及防火门检查记录");
        // 设置列宽
        sheet.setColumnWidth(0, 1200);
        sheet.setColumnWidth(1, 3500);
        sheet.setColumnWidth(2, 3500);
        sheet.setColumnWidth(3, 4500);
        sheet.setColumnWidth(4, 4500);
        sheet.setColumnWidth(5, 5500);
        sheet.setColumnWidth(6, 4500);
        sheet.setColumnWidth(7, 4500);
        sheet.setColumnWidth(8, 4500);
        //第一行
        XSSFRow row1 = sheet.createRow(0);
        row1.createCell(0).setCellValue("CBP-6.3.0-SCFSH-020.1" + val + "  Rev:04  2021/10/01");
        ExcelUtils.createVoidCell(1, 8, row1);
        sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 8));//合并
        //第二行
        XSSFRow row2 = sheet.createRow(1);
        row2.createCell(0).setCellValue(tFireDoor.getDevType()+"常闭式防爆门/防火门检查记录");
        ExcelUtils.createVoidCell(1, 8, row2);
        sheet.addMergedRegion(new CellRangeAddress(1, 1, 0, 8));
        //第三行
        XSSFRow row3 = sheet.createRow(2);
        row3.createCell(0).setCellValue("年度：" + tFireDoor.getYear());
        sheet.addMergedRegion(new CellRangeAddress(2, 2, 0, 1));
        ExcelUtils.createVoidCell(1, 8, row3);
        row3.createCell(2).setCellValue("检查频率：1次/月");
        sheet.addMergedRegion(new CellRangeAddress(2, 2, 2, 8));
        //第四行
        XSSFRow row4 = sheet.createRow(3);
        row4.createCell(0).setCellValue("序号");
        row4.createCell(1).setCellValue("位号");
        row4.createCell(2).setCellValue("检查内容");
        ExcelUtils.createVoidCell(4, 8, row4);
        sheet.addMergedRegion(new CellRangeAddress(3, 3, 2, 8));
        //第五行
        XSSFRow row5 = sheet.createRow(4);
        sheet.addMergedRegion(new CellRangeAddress(3, 4, 0, 0));
        sheet.addMergedRegion(new CellRangeAddress(3, 4, 1, 1));
        ExcelUtils.createVoidCell(0, 2, row5);
        row5.createCell(2).setCellValue("防爆门/防火门关闭状态");
        row5.createCell(3).setCellValue("防爆门/防火门密闭性能");
        row5.createCell(4).setCellValue("防爆门/防火门外观");
        row5.createCell(5).setCellValue("防爆门/防火门润滑性能");
        row5.createCell(6).setCellValue("防爆门/防火门闭门器外观");
        row5.createCell(7).setCellValue("防爆门/防火门闭门器性能");
        row5.createCell(8).setCellValue("检查人");
        Long planId = null;
        //遍历数组赋值
        for (int i = 0; i < list.size(); i++) {
            TFireDoor door = list.get(i);
            planId = door.getPlanId();
            XSSFRow row = sheet.createRow(5 + i);
            row.createCell(0).setCellValue(i + 1);
            row.createCell(1).setCellValue(door.getDevNo());
            row.createCell(2).setCellValue("1".equals(door.getInactiveState()) ? "√" : door.getInactiveState());
            row.createCell(3).setCellValue("1".equals(door.getSealingProperty()) ? "√" : door.getSealingProperty());
            row.createCell(4).setCellValue("1".equals(door.getAppearance()) ? "√" : door.getAppearance());
            row.createCell(5).setCellValue("1".equals(door.getLubricatingProperty()) ? "√" : door.getLubricatingProperty());
            row.createCell(6).setCellValue("1".equals(door.getDoorClosers()) ? "√" : door.getDoorClosers());
            row.createCell(7).setCellValue("1".equals(door.getPerformanceClosers()) ? "√" : door.getPerformanceClosers());
            XSSFCell cell10 = row.createCell(8);
            if (StringUtils.isNotEmpty(door.getUpdaterCode())) {
                SysUser sysUser = sysUserService.selectUserById(Long.valueOf(door.getUpdaterCode()));
                cell10.setCellValue(sysUser == null ? "" : sysUser.getNickName());
            }
        }
        String confirmed = "";
        Date confirmedDate = null;
        TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(planId);
        if (Objects.nonNull(tPatrolPlan) && Objects.nonNull(tPatrolPlan.getConfirmed())) {
            SysUser sysUser = sysUserService.selectUserById(Long.valueOf(tPatrolPlan.getConfirmed()));            if (sysUser!=null) {
                confirmed = sysUser.getSignUrl();
                confirmedDate = tPatrolPlan.getConfirmDate();
            }
        }
        // 循环后下一行
        int nextRowNum = 5 + list.size();
        XSSFRow row28 = sheet.createRow(nextRowNum);
        row28.setHeightInPoints(30);
        XSSFCell row28Cell1 = row28.createCell(0);
        row28Cell1.setCellValue("签名");
        ExcelUtils.createVoidCell(1, 2, row28);
        sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
        if (StringUtils.isNotEmpty(confirmed))
            ExcelUtils.insertPicture(workbook, sheet, confirmed, nextRowNum, 5, 1, 1);
        ExcelUtils.createVoidCell(2, 8, row28);
        sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 8));
        // 下一行
        nextRowNum++;
        XSSFRow row29 = sheet.createRow(nextRowNum);
        row29.setHeightInPoints(30);
        XSSFCell row29Cell1 = row29.createCell(0);
        row29Cell1.setCellValue("日期");
        ExcelUtils.createVoidCell(1, 1, row29);
        sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 1));
        row29.createCell(2).setCellValue(confirmedDate != null ? new SimpleDateFormat("yyyy-MM-dd").format(confirmedDate) : "");
        ExcelUtils.createVoidCell(3, 8, row29);
        sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 2, 8));
    }

    /**
     * 获取常闭式防爆门/防火门详细信息
     */
    @PreAuthorize("@ss.hasPermi('fire:door:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tFireDoorService.selectTFireDoorById(id));
    }

    /**
     * 获取常闭式防爆门/防火门详细信息
     */
    @PreAuthorize("@ss.hasPermi('fire:door:query')")
    @GetMapping(value = "/info")
    public AjaxResult getInfo(TFireDoor tFireDoor) {
        if (StringUtils.isEmpty(tFireDoor.getDevType()) || StringUtils.isEmpty(tFireDoor.getDevNo()) || StringUtils.isNull(tFireDoor.getPlanId())) {
            return AjaxResult.error("防火门归属或x位号或计划id不能为空！");
        }
        TFireDoor item = new TFireDoor();
        item.setPlanId(tFireDoor.getPlanId());
        item.setDevNo(tFireDoor.getDevNo());
        item.setDevType(tFireDoor.getDevType());
        item.setIssuesFlag("0");
        List<TFireDoor> list = tFireDoorService.selectTFireDoor(item);
        if (CollectionUtils.isNotEmpty(list))
            return AjaxResult.success(list.get(0));
        return AjaxResult.error("此设备不在当前巡检计划中!");
    }

    /**
     * 新增常闭式防爆门/防火门
     */
    @PreAuthorize("@ss.hasPermi('fire:door:add')")
    @Log(title = "常闭式防爆门/防火门", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TFireDoor tFireDoor) {
        return toAjax(tFireDoorService.insertTFireDoor(tFireDoor));
    }

    /**
     * 修改常闭式防爆门/防火门
     */
    @PreAuthorize("@ss.hasPermi('fire:door:edit')")
    @Log(title = "常闭式防爆门/防火门", businessType = BusinessType.UPDATE)
    @PutMapping
    @RepeatSubmit
    public AjaxResult edit(@RequestBody TFireDoor tFireDoor) {
        tFireDoor.setUpdatedate(new Date());
        tFireDoor.setUpdaterCode(getUserId().toString());
        tFireDoor.setCheckStatus("1");
        tFireDoor.setCheckDate(new Date());
        TFireDoor item = tFireDoorService.selectTFireDoorById(tFireDoor.getId());
        TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(tFireDoor.getPlanId());
        if (tPatrolPlan.getStatus()==1){
            return AjaxResult.error("当前选择的计划已确认！");
        }
        if (!"1".equals(item.getCheckStatus())) {
            tPatrolPlan.setUncheckedNum(tPatrolPlan.getUncheckedNum() - 1);
            tPatrolPlan.setCheckedNum(tPatrolPlan.getCheckedNum() + 1);
            tPatrolPlanService.updateTPatrolPlan(tPatrolPlan);
        }
        if ((StringUtils.isNotEmpty(tFireDoor.getDoorClosers()) && !"1".equals(tFireDoor.getDoorClosers())) ||
                (StringUtils.isNotEmpty(tFireDoor.getAppearance()) && !"1".equals(tFireDoor.getAppearance())) ||
                (StringUtils.isNotEmpty(tFireDoor.getInactiveState()) && !"1".equals(tFireDoor.getInactiveState())) ||
                (StringUtils.isNotEmpty(tFireDoor.getLubricatingProperty()) && !"1".equals(tFireDoor.getLubricatingProperty())) ||
                (StringUtils.isNotEmpty(tFireDoor.getPerformanceClosers()) && !"1".equals(tFireDoor.getPerformanceClosers())) ||
                (StringUtils.isNotEmpty(tFireDoor.getSealingProperty()) && !"1".equals(tFireDoor.getSealingProperty()))) {
            tFireDoor.setIssuesStatus("1");
            tFireDoor.setCheckStatus("0");
            tFireDoorService.updateTFireDoor(tFireDoor);
            tFireDoor.setIssuesFlag("1");
            tFireDoor.setCheckStatus("1");
            tFireDoor.setCreatedate(new Date());
            tFireDoor.setCreaterCode(getUserId().toString());
            tFireDoor.setDevNo(item.getDevNo());
            tFireDoorService.insertTFireDoor(tFireDoor);
            TPatrolIssues tPatrolIssues = new TPatrolIssues();
            tPatrolIssues.setPatrolType("14");
            tPatrolIssues.setDevDescribe(tFireDoor.getDevType() + "常闭式防爆门/防火门_" + tFireDoor.getDevNo());
            tPatrolIssues.setLinkId(tFireDoor.getId());
            tPatrolIssues.setStatus(0L);
            tPatrolIssuesService.insertTPatrolIssues(tPatrolIssues);
        } else {
            if ("1".equals(item.getIssuesStatus())) {
                TPatrolIssues tPatrolIssues = new TPatrolIssues();
                tPatrolIssues.setPatrolType("14");
                tPatrolIssues.setDevDescribe(item.getDevType() + "常闭式防爆门/防火门_" + item.getDevNo());
                tPatrolIssues.setCreatedate(new Date());
                for (TPatrolIssues patrolIssues : tPatrolIssuesService.selectTPatrolIssuesList(tPatrolIssues)) {
                    patrolIssues.setStatus(1L);
                    tPatrolIssuesService.updateTPatrolIssues(patrolIssues);
                }
            }
            tFireDoor.setIssuesStatus("0");
            tFireDoorService.updateTFireDoor(tFireDoor);
        }
        return AjaxResult.success();
    }

    /**
     * 删除常闭式防爆门/防火门
     */
    @PreAuthorize("@ss.hasPermi('fire:door:remove')")
    @Log(title = "常闭式防爆门/防火门", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tFireDoorService.deleteTFireDoorByIds(ids));
    }
}
