package com.ruoyi.project.plant.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.plant.domain.TAwardrecord;
import com.ruoyi.project.plant.service.ITAwardrecordService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 人员嘉奖记录Controller
 *
 * @author ruoyi
 * @date 2020-12-08
 */
@RestController
@RequestMapping("/plant/awardrecord")
public class TAwardrecordController extends BaseController
{
    @Autowired
    private ITAwardrecordService tAwardrecordService;

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    /**
     * 查询人员嘉奖记录列表
     */
    @PreAuthorize("@ss.hasPermi('plant:awardrecord:list')")
    @GetMapping("/list")
    public TableDataInfo list(TAwardrecord tAwardrecord)
    {
        startPage();
        List<TAwardrecord> list = tAwardrecordService.selectTAwardrecordList(tAwardrecord);
        return getDataTable(list);
    }

    /**
     * 导出人员嘉奖记录列表
     */
    @PreAuthorize("@ss.hasPermi('plant:awardrecord:export')")
    @Log(title = "人员嘉奖记录", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TAwardrecord tAwardrecord)
    {
        List<TAwardrecord> list = tAwardrecordService.selectTAwardrecordList(tAwardrecord);
        ExcelUtil<TAwardrecord> util = new ExcelUtil<TAwardrecord>(TAwardrecord.class);
        return util.exportExcel(list, "awardrecord");
    }

    /**
     * 获取人员嘉奖记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('plant:awardrecord:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tAwardrecordService.selectTAwardrecordById(id));
    }

    /**
     * 新增人员嘉奖记录
     */
    @PreAuthorize("@ss.hasPermi('plant:awardrecord:add')")
    @Log(title = "人员嘉奖记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TAwardrecord tAwardrecord)
    {
        tAwardrecord.setCreaterCode(getUserId().toString());
        return toAjax(tAwardrecordService.insertTAwardrecord(tAwardrecord));
    }

    /**
     * 修改人员嘉奖记录
     */
    @PreAuthorize("@ss.hasPermi('plant:awardrecord:edit')")
    @Log(title = "人员嘉奖记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TAwardrecord tAwardrecord)
    {
        tAwardrecord.setUpdaterCode(getUserId().toString());
        tAwardrecord.setUpdatedate(new Date());
        return toAjax(tAwardrecordService.updateTAwardrecord(tAwardrecord));
    }

    /**
     * 删除人员嘉奖记录
     */
    @PreAuthorize("@ss.hasPermi('plant:awardrecord:remove')")
    @Log(title = "人员嘉奖记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tAwardrecordService.deleteTAwardrecordByIds(ids));
    }

    /**
     * 批量导入人员嘉奖记录
     */
    @PreAuthorize("@ss.hasPermi('plant:awardrecord:add')")
    @Log(title = "人员嘉奖记录", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TAwardrecord> list = new ArrayList<TAwardrecord>();
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        List<SysDictData> team = iSysDictTypeService.selectDictDataByType("TEAM_DIVIDE");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TAwardrecord entity = new TAwardrecord();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        entity.setStaffid(cellValue);//员工编号
                    } else if (j == 2) {
                        entity.setName(cellValue);//员工姓名
                    } else if (j == 3) {
                        for (SysDictData p : team) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setTeam(p.getDictValue());//班值
                            }
                        }
                    } else if (j == 4) {
                        entity.setCommendation(cellValue);//嘉奖记录
                    } else if (j == 5) {
                        if (cellValue.length() > 3) {
                            entity.setCommendationdate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//嘉奖日期
                        }
                    } else if (j == 6) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 7) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TAwardrecord t : list
        ) {
            failNum++;
            try {
                tAwardrecordService.insertTAwardrecord(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
