package com.ruoyi.project.production.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TLbnhjstj;
import com.ruoyi.project.production.service.ITLbnhjstjService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 能耗管理统计Controller
 *
 * @author ruoyi
 * @date 2020-12-14
 */
@RestController
@RequestMapping("/production/lbnhjstj")
public class TLbnhjstjController extends BaseController
{
    @Autowired
    private ITLbnhjstjService tLbnhjstjService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    @Autowired
    private ISysDeptService iSysDeptService;

    /**
     * 查询能耗管理统计列表
     */
    @PreAuthorize("@ss.hasPermi('production:lbnhjstj:list')")
    @GetMapping("/list")
    public TableDataInfo list(TLbnhjstj tLbnhjstj)
    {
        startPage();
        List<TLbnhjstj> list = tLbnhjstjService.selectTLbnhjstjList(tLbnhjstj);
        return getDataTable(list);
    }

    /**
     * 导出能耗管理统计列表
     */
    @PreAuthorize("@ss.hasPermi('production:lbnhjstj:export')")
    @Log(title = "能耗管理统计", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TLbnhjstj tLbnhjstj)
    {
        List<TLbnhjstj> list = tLbnhjstjService.selectTLbnhjstjList(tLbnhjstj);
        ExcelUtil<TLbnhjstj> util = new ExcelUtil<TLbnhjstj>(TLbnhjstj.class);
        return util.exportExcel(list, "lbnhjstj");
    }

    /**
     * 获取能耗管理统计详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:lbnhjstj:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tLbnhjstjService.selectTLbnhjstjById(id));
    }

    /**
     * 新增能耗管理统计
     */
    @PreAuthorize("@ss.hasPermi('production:lbnhjstj:add')")
    @Log(title = "能耗管理统计", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TLbnhjstj tLbnhjstj)
    {
        tLbnhjstj.setCreaterCode(getUserId().toString());
        return toAjax(tLbnhjstjService.insertTLbnhjstj(tLbnhjstj));
    }

    /**
     * 修改能耗管理统计
     */
    @PreAuthorize("@ss.hasPermi('production:lbnhjstj:edit')")
    @Log(title = "能耗管理统计", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TLbnhjstj tLbnhjstj)
    {
        tLbnhjstj.setUpdaterCode(getUserId().toString());
        tLbnhjstj.setUpdatedate(new Date());
        return toAjax(tLbnhjstjService.updateTLbnhjstj(tLbnhjstj));
    }

    /**
     * 删除能耗管理统计
     */
    @PreAuthorize("@ss.hasPermi('production:lbnhjstj:remove')")
    @Log(title = "能耗管理统计", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tLbnhjstjService.deleteTLbnhjstjByIds(ids));
    }

    /**
     * 批量导入能耗管理统计
     */
    @PreAuthorize("@ss.hasPermi('production:lbnhjstj:add')")
    @Log(title = "能耗管理统计", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TLbnhjstj> list = new ArrayList<TLbnhjstj>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TLbnhjstj entity = new TLbnhjstj();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        if (cellValue.length() > 3) {
                            entity.setStarttime(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//开始时间
                        }
                    } else if (j == 2) {
                        if (cellValue.length() > 3) {
                            entity.setEndtime(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//结束时间
                        }
                    } else if (j == 3) {
                        entity.setTeam(cellValue);//班组
                    } else if (j == 4) {
                        if (cellValue != ""){
                            entity.setCorrecenergysaving(Long.parseLong(cellValue));//校正节能量(100kgce/班次)
                        }
                    } else if (j == 5) {
                        if (cellValue != ""){
                            entity.setEthyleneprocd(Long.parseLong(cellValue));//乙烯产量(吨/班次)
                        }
                    } else if (j == 6) {
                        if (cellValue != ""){
                            entity.setEthyenergyconsump(Long.parseLong(cellValue));//乙烯能耗(班次)
                        }
                    } else if (j == 7) {
                        if (cellValue != ""){
                            entity.setBtxprocd(Long.parseLong(cellValue));//芳烃产量(吨/班次)
                        }
                    } else if (j == 8) {
                        if (cellValue != ""){
                            entity.setBtxenergyconsump(Long.parseLong(cellValue));//芳烃能耗(班次)
                        }
                    } else if (j == 9) {
                        entity.setRemarks(cellValue);//备注
                    } else if (j == 10) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TLbnhjstj t : list
        ) {
            failNum++;
            try {
                tLbnhjstjService.insertTLbnhjstj(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
