package com.ruoyi.project.production.controller;

import java.util.Date;
import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TMonthlyQualityReportS3022;
import com.ruoyi.project.production.service.ITMonthlyQualityReportS3022Service;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 质量月报S3022Controller
 *
 * @author ruoyi
 * @date 2023-10-27
 */
@RestController
@RequestMapping("/production/s3022")
public class TMonthlyQualityReportS3022Controller extends BaseController
{
    @Autowired
    private ITMonthlyQualityReportS3022Service tMonthlyQualityReportS3022Service;

    /**
     * 查询质量月报S3022列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:list')")
    @GetMapping("/list")
    public AjaxResult list(TMonthlyQualityReportS3022 tMonthlyQualityReportS3022)
    {
        Date endDate = tMonthlyQualityReportS3022.getEndDate();
        endDate.setHours(23);
        endDate.setMinutes(59);
        endDate.setSeconds(59);
        tMonthlyQualityReportS3022.setEndDate(endDate);
        return AjaxResult.success(tMonthlyQualityReportS3022Service.selectTMonthlyQualityReportS3022List(tMonthlyQualityReportS3022));
    }

    /**
     * 导出质量月报S3022列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:export')")
    @Log(title = "质量月报S3022", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TMonthlyQualityReportS3022 tMonthlyQualityReportS3022)
    {
        List<TMonthlyQualityReportS3022> list = tMonthlyQualityReportS3022Service.selectTMonthlyQualityReportS3022List(tMonthlyQualityReportS3022);
        ExcelUtil<TMonthlyQualityReportS3022> util = new ExcelUtil<TMonthlyQualityReportS3022>(TMonthlyQualityReportS3022.class);
        return util.exportExcel(list, "quality");
    }

    /**
     * 获取质量月报S3022详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:quality:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tMonthlyQualityReportS3022Service.selectTMonthlyQualityReportS3022ById(id));
    }

    /**
     * 新增质量月报S3022
     */
    @PreAuthorize("@ss.hasPermi('production:quality:add')")
    @Log(title = "质量月报S3022", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TMonthlyQualityReportS3022 tMonthlyQualityReportS3022)
    {
        return toAjax(tMonthlyQualityReportS3022Service.insertTMonthlyQualityReportS3022(tMonthlyQualityReportS3022));
    }

    /**
     * 修改质量月报S3022
     */
    @PreAuthorize("@ss.hasPermi('production:quality:edit')")
    @Log(title = "质量月报S3022", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TMonthlyQualityReportS3022 tMonthlyQualityReportS3022)
    {
        return toAjax(tMonthlyQualityReportS3022Service.updateTMonthlyQualityReportS3022(tMonthlyQualityReportS3022));
    }

    /**
     * 删除质量月报S3022
     */
    @PreAuthorize("@ss.hasPermi('production:quality:remove')")
    @Log(title = "质量月报S3022", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tMonthlyQualityReportS3022Service.deleteTMonthlyQualityReportS3022ByIds(ids));
    }
}
