package com.ruoyi.project.production.controller;

import java.util.Date;
import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TMonthlyQualityReportS5010;
import com.ruoyi.project.production.service.ITMonthlyQualityReportS5010Service;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 质量月报S5010Controller
 *
 * @author ruoyi
 * @date 2023-10-27
 */
@RestController
@RequestMapping("/production/s5010")
public class TMonthlyQualityReportS5010Controller extends BaseController
{
    @Autowired
    private ITMonthlyQualityReportS5010Service tMonthlyQualityReportS5010Service;

    /**
     * 查询质量月报S5010列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:list')")
    @GetMapping("/list")
    public AjaxResult list(TMonthlyQualityReportS5010 tMonthlyQualityReportS5010)
    {
        Date endDate = tMonthlyQualityReportS5010.getEndDate();
        endDate.setHours(23);
        endDate.setMinutes(59);
        endDate.setSeconds(59);
        tMonthlyQualityReportS5010.setEndDate(endDate);
        return AjaxResult.success(tMonthlyQualityReportS5010Service.selectTMonthlyQualityReportS5010List(tMonthlyQualityReportS5010));
    }

    /**
     * 导出质量月报S5010列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:export')")
    @Log(title = "质量月报S5010", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TMonthlyQualityReportS5010 tMonthlyQualityReportS5010)
    {
        List<TMonthlyQualityReportS5010> list = tMonthlyQualityReportS5010Service.selectTMonthlyQualityReportS5010List(tMonthlyQualityReportS5010);
        ExcelUtil<TMonthlyQualityReportS5010> util = new ExcelUtil<TMonthlyQualityReportS5010>(TMonthlyQualityReportS5010.class);
        return util.exportExcel(list, "quality");
    }

    /**
     * 获取质量月报S5010详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:quality:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tMonthlyQualityReportS5010Service.selectTMonthlyQualityReportS5010ById(id));
    }

    /**
     * 新增质量月报S5010
     */
    @PreAuthorize("@ss.hasPermi('production:quality:add')")
    @Log(title = "质量月报S5010", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TMonthlyQualityReportS5010 tMonthlyQualityReportS5010)
    {
        return toAjax(tMonthlyQualityReportS5010Service.insertTMonthlyQualityReportS5010(tMonthlyQualityReportS5010));
    }

    /**
     * 修改质量月报S5010
     */
    @PreAuthorize("@ss.hasPermi('production:quality:edit')")
    @Log(title = "质量月报S5010", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TMonthlyQualityReportS5010 tMonthlyQualityReportS5010)
    {
        return toAjax(tMonthlyQualityReportS5010Service.updateTMonthlyQualityReportS5010(tMonthlyQualityReportS5010));
    }

    /**
     * 删除质量月报S5010
     */
    @PreAuthorize("@ss.hasPermi('production:quality:remove')")
    @Log(title = "质量月报S5010", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tMonthlyQualityReportS5010Service.deleteTMonthlyQualityReportS5010ByIds(ids));
    }
}
