package com.ruoyi.project.production.controller;

import java.util.Date;
import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TMonthlyQualityReportZ404;
import com.ruoyi.project.production.service.ITMonthlyQualityReportZ404Service;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 质量月报Z404Controller
 *
 * @author ruoyi
 * @date 2023-10-27
 */
@RestController
@RequestMapping("/production/z404")
public class TMonthlyQualityReportZ404Controller extends BaseController
{
    @Autowired
    private ITMonthlyQualityReportZ404Service tMonthlyQualityReportZ404Service;

    /**
     * 查询质量月报Z404列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:list')")
    @GetMapping("/list")
    public AjaxResult list(TMonthlyQualityReportZ404 tMonthlyQualityReportZ404)
    {
        Date endDate = tMonthlyQualityReportZ404.getEndDate();
        endDate.setHours(23);
        endDate.setMinutes(59);
        endDate.setSeconds(59);
        tMonthlyQualityReportZ404.setEndDate(endDate);
        return AjaxResult.success(tMonthlyQualityReportZ404Service.selectTMonthlyQualityReportZ404List(tMonthlyQualityReportZ404));
    }

    /**
     * 导出质量月报Z404列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:export')")
    @Log(title = "质量月报Z404", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TMonthlyQualityReportZ404 tMonthlyQualityReportZ404)
    {
        List<TMonthlyQualityReportZ404> list = tMonthlyQualityReportZ404Service.selectTMonthlyQualityReportZ404List(tMonthlyQualityReportZ404);
        ExcelUtil<TMonthlyQualityReportZ404> util = new ExcelUtil<TMonthlyQualityReportZ404>(TMonthlyQualityReportZ404.class);
        return util.exportExcel(list, "z404");
    }

    /**
     * 获取质量月报Z404详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:quality:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tMonthlyQualityReportZ404Service.selectTMonthlyQualityReportZ404ById(id));
    }

    /**
     * 新增质量月报Z404
     */
    @PreAuthorize("@ss.hasPermi('production:quality:add')")
    @Log(title = "质量月报Z404", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TMonthlyQualityReportZ404 tMonthlyQualityReportZ404)
    {
        return toAjax(tMonthlyQualityReportZ404Service.insertTMonthlyQualityReportZ404(tMonthlyQualityReportZ404));
    }

    /**
     * 修改质量月报Z404
     */
    @PreAuthorize("@ss.hasPermi('production:quality:edit')")
    @Log(title = "质量月报Z404", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TMonthlyQualityReportZ404 tMonthlyQualityReportZ404)
    {
        return toAjax(tMonthlyQualityReportZ404Service.updateTMonthlyQualityReportZ404(tMonthlyQualityReportZ404));
    }

    /**
     * 删除质量月报Z404
     */
    @PreAuthorize("@ss.hasPermi('production:quality:remove')")
    @Log(title = "质量月报Z404", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tMonthlyQualityReportZ404Service.deleteTMonthlyQualityReportZ404ByIds(ids));
    }
}
