package com.ruoyi.project.production.mapper;

import java.util.List;
import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.production.controller.vo.DailyAnalysisDataVO;
import com.ruoyi.project.production.domain.TDailyProductionReport;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

/**
 * 每日生产报告Mapper接口
 * 
 * @author ruoyi
 * @date 2022-08-23
 */
public interface TDailyProductionReportMapper
{
    /**
     * 查询日期最新的每日生产报告
     *
     * @return 每日生产报告
     */
    public TDailyProductionReport selectTDailyProductionReportByMaxDate();

    /**
     * 查询每日生产报告
     * 
     * @param id 每日生产报告ID
     * @return 每日生产报告
     */
    public TDailyProductionReport selectTDailyProductionReportById(Long id);

    /**
     * 查询每日生产报告列表
     * 
     * @param tDailyProductionReport 每日生产报告
     * @return 每日生产报告集合
     */
    @DataScope(deptAlias = "d")
    public List<TDailyProductionReport> selectTDailyProductionReportList(TDailyProductionReport tDailyProductionReport);
    public List<TDailyProductionReport> selectTDailyProductionReportListMonthly(TDailyProductionReport tDailyProductionReport);

    /**
     * 新增每日生产报告
     * 
     * @param tDailyProductionReport 每日生产报告
     * @return 结果
     */
    public int insertTDailyProductionReport(TDailyProductionReport tDailyProductionReport);

    /**
     * 修改每日生产报告
     * 
     * @param tDailyProductionReport 每日生产报告
     * @return 结果
     */
    public int updateTDailyProductionReport(TDailyProductionReport tDailyProductionReport);

    /**
     * 根据日期修改每日生产报告
     *
     * @param tDailyProductionReport 每日生产报告
     * @return 结果
     */
    public int updateTDailyProductionReportByReportDate(TDailyProductionReport tDailyProductionReport);

    /**
     * 删除每日生产报告
     * 
     * @param id 每日生产报告ID
     * @return 结果
     */
    public int deleteTDailyProductionReportById(Long id);

    /**
     * 批量删除每日生产报告
     * 
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTDailyProductionReportByIds(Long[] ids);

    @Select("select d.${fieldName} as valueData, d.report_date as dateData " +
            "from t_daily_production_report d " +
            "where d.REPORT_DATE >= to_date('${startDate}', 'yyyy-mm-dd') " +
            "and d.REPORT_DATE <= to_date('${endDate}', 'yyyy-mm-dd') " +
            "order by d.report_date")
    public List<DailyAnalysisDataVO> selectAnalysisData(@Param("fieldName") String fieldName,
                                                 @Param("startDate") String startDate,
                                                 @Param("endDate") String endDate);

    public TDailyProductionReport selectDailySum(@Param("reportYear") int reportYear,
                                            @Param("reportMonth") int reportMonth);
}
