package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.pssr.domain.TPssrApprove;
import com.ruoyi.project.pssr.domain.TPssrCircuit;
import com.ruoyi.project.pssr.domain.TPssrSubcontent;
import com.ruoyi.project.pssr.domain.TPssrTurndown;
import com.ruoyi.project.pssr.mapper.TPssrCircuitMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.ProcessEngine;
import org.activiti.engine.ProcessEngines;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 短接Controller
 *
 * @author ssy
 * @date 2024-09-18
 */
@RestController
@RequestMapping("/pssr/circuit")
public class TPssrCircuitController extends BaseController {
    @Autowired
    private ITPssrFileService tPssrFileService;

    @Resource
    private TPssrCircuitMapper tPssrCircuitMapper;

    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrCircuitService tPssrCircuitService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "dj";

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysUserService userService;

    /**
     * 批量导入
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:add')")
    @PostMapping("/importData")
    public AjaxResult importInterlockData(MultipartFile file, Long subId) throws IOException {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow = new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPssrCircuit> list = new ArrayList<TPssrCircuit>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 2; i <= rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPssrCircuit entity = new TPssrCircuit();
                entity.setDeptId(userService.selectUserById(getUserId()).getDeptId());
                entity.setSubId(subId);
                entity.setApproveStatus(0L);
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        entity.setUnit(cellValue);
                    } else if (j == 1) {
                        entity.setPidNo(cellValue);
                    } else if (j == 2) {
                        entity.setPipelineNo(cellValue);
                    } else if (j == 3) {
                        entity.setCircuitType(cellValue);
                    } else if (j == 4) {
                        entity.setMedium(cellValue);
                    } else if (j == 5) {
                        entity.setPosition(cellValue);
                    } else if (j == 6) {
                        entity.setCircuitSize(cellValue);
                    } else if (j == 7) {
                        entity.setRightStatus(cellValue);
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            } catch (Exception e) {
                failNumber++;
                failRow.add(i + 1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPssrCircuit t : list
        ) {
            failNum++;
            try {
                this.add(t);
                successNumber++;
            } catch (Exception e) {
                failNumber++;
                logger.info("e:" + e);
                failRow.add(failNum + 1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" + String.valueOf(successNumber));
        logger.info("failNumber:" + String.valueOf(failNumber));
        logger.info("failRow:" + String.valueOf(failRow));
        return AjaxResult.success("导入成功行数：" + String.valueOf(successNumber));
    }

    /**
     * 查询短接列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrCircuit tPssrCircuit) {
        try {
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(tPssrCircuit.getSubId());
            if (approve != null) {
                ProcessEngine processEngine = ProcessEngines.getDefaultProcessEngine();
                TaskService taskService = processEngine.getTaskService();
                Task task = processEngine.getTaskService()//获取任务service
                        .createTaskQuery()//创建查询对象
                        .taskAssignee(getUserId().toString())
                        .processInstanceId(approve.getProcessId()).singleResult();
                if (task != null) {
                    if (task.getName().equals("确认人1")){
                        tPssrCircuit.setConfirm1(getUserId().toString());
                    }else if (task.getName().equals("确认人2")){
                       tPssrCircuit.setConfirm2(getUserId().toString());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("待办确认人查询报错：{}",e.getMessage());
        }
        startPage();
        List<TPssrCircuit> list = tPssrCircuitService.selectTPssrCircuitList(tPssrCircuit);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "dj"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "dj"));
        });
        return getDataTable(list);
    }

    /**
     * 导出短接列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:export')")
    @Log(title = "短接", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrCircuit tPssrCircuit) {
        List<TPssrCircuit> list = tPssrCircuitService.selectTPssrCircuitList(tPssrCircuit);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrCircuit> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/dj.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 3;
            int num = 1;

            Row originalRow = sheet.getRow(3);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrCircuit t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getUnit());
                row.createCell(2).setCellValue(t.getPidNo());
                row.createCell(3).setCellValue(t.getPipelineNo());
                row.createCell(4).setCellValue(t.getCircuitType());
                row.createCell(5).setCellValue(t.getMedium());
                row.createCell(6).setCellValue(t.getPosition());
                row.createCell(7).setCellValue(t.getCircuitSize());
                row.createCell(8).setCellValue(t.getRightStatus());
                row.createCell(9).setCellValue(t.getCurrentStatus());
                row.createCell(10);
                row.createCell(11).setCellValue(t.getCurrentStatusRight());
                row.createCell(12);
                try {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(t.getConfirm1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirm2()));
                    String confirm1 = sysUser.getSignUrl();
                    String confirm2 = sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 10, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 12, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(13).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(14).setCellValue(t.getRemarks());


                //渲染样式
                for (int i = 0; i < 15; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename = "PSSR_11_短接" + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取短接详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrCircuit item = tPssrCircuitService.selectTPssrCircuitById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "dj"));
        if (item.getApproveStatus() != 2)
            item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "dj"));
        return AjaxResult.success(item);
    }

    /**
     * 新增短接
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:add')")
    @Log(title = "短接", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrCircuit tPssrCircuit) {
        if (StringUtils.isNotEmpty(tPssrCircuit.getConfirm1()) && tPssrCircuit.getConfirm1().equals(tPssrCircuit.getConfirm2())) {
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        if (StringUtils.isNotEmpty(tPssrCircuit.getRightStatus()) && StringUtils.isEmpty(tPssrCircuit.getCurrentStatus())) {
            tPssrCircuit.setCurrentStatus(tPssrCircuit.getRightStatus());
            tPssrCircuit.setCurrentStatusRight("是");
        } else if (StringUtils.isNotEmpty(tPssrCircuit.getRightStatus()) && StringUtils.isNotEmpty(tPssrCircuit.getCurrentStatus())) {
            if (tPssrCircuit.getRightStatus().equals(tPssrCircuit.getCurrentStatus())) {
                tPssrCircuit.setCurrentStatusRight("是");
            } else {
                tPssrCircuit.setCurrentStatusRight("否");
            }
        } else if (StringUtils.isEmpty(tPssrCircuit.getRightStatus()) && StringUtils.isNotEmpty(tPssrCircuit.getCurrentStatus())) {
            tPssrCircuit.setRightStatus(tPssrCircuit.getCurrentStatus());
            tPssrCircuit.setCurrentStatusRight("是");
        } else {
            tPssrCircuit.setCurrentStatusRight("否");
        }
        tPssrCircuit.setApproveStatus(0L);
        return toAjax(tPssrCircuitService.insertTPssrCircuit(tPssrCircuit));
    }

    /**
     * 修改短接
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:edit')")
    @Log(title = "短接", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrCircuit tPssrCircuit) {
        if (tPssrCircuit.getConfirm1().equals(tPssrCircuit.getConfirm2())) {
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        if (StringUtils.isNotEmpty(tPssrCircuit.getRightStatus()) && StringUtils.isEmpty(tPssrCircuit.getCurrentStatus())) {
            tPssrCircuit.setCurrentStatus(tPssrCircuit.getRightStatus());
            tPssrCircuit.setCurrentStatusRight("是");
        } else if (StringUtils.isNotEmpty(tPssrCircuit.getRightStatus()) && StringUtils.isNotEmpty(tPssrCircuit.getCurrentStatus())) {
            if (tPssrCircuit.getRightStatus().equals(tPssrCircuit.getCurrentStatus())) {
                tPssrCircuit.setCurrentStatusRight("是");
            } else {
                tPssrCircuit.setCurrentStatusRight("否");
            }
        } else if (StringUtils.isEmpty(tPssrCircuit.getRightStatus()) && StringUtils.isNotEmpty(tPssrCircuit.getCurrentStatus())) {
            tPssrCircuit.setRightStatus(tPssrCircuit.getCurrentStatus());
            tPssrCircuit.setCurrentStatusRight("是");
        } else {
            tPssrCircuit.setCurrentStatusRight("否");
        }
        tPssrFileService.updateFileRelevance(tPssrCircuit.getFileIds(), "dj", tPssrCircuit.getId(), tPssrCircuit.getSubId());
        return toAjax(tPssrCircuitService.updateTPssrCircuit(tPssrCircuit));
    }

    /**
     * 修改短接
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:edit')")
    @Log(title = "短接", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editb(@RequestBody TPssrCircuit tPssrCircuit) {
        if (tPssrCircuit.getConfirm1().equals(tPssrCircuit.getConfirm2())) {
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        return toAjax(tPssrCircuitMapper.updateTPssrCircuitByIds(tPssrCircuit));
    }

    /**
     * 删除短接
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:remove')")
    @Log(title = "短接", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrCircuitService.deleteTPssrCircuitByIds(ids));
    }

    /**
     * 确认短接
     */
    @PreAuthorize("@ss.hasPermi('pssr:circuit:edit')")
    @Log(title = "短接", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmCircuit")
    public AjaxResult confirmCircuit(@RequestBody TPssrCircuit tPssrCircuit) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        TPssrCircuit lock = new TPssrCircuit();
        if (tPssrCircuit.getTaskType() == 4) {
            //拆锁确认
            queryStatus = 1;
            approveStatus = 3;
            date = new Date();
            lock.setConfirm1(getUserId().toString());
        } else if (tPssrCircuit.getTaskType() == 5) {
            //上锁确认
            queryStatus = 3;
            approveStatus = 2;
            date = new Date();
            lock.setConfirm2(getUserId().toString());
        }
        // 修改状态
        if (tPssrCircuit.getIds() != null && tPssrCircuit.getIds().length > 0) {
            for (Long id : tPssrCircuit.getIds()) {
                TPssrCircuit item = tPssrCircuitService.selectTPssrCircuitById(id);
                item.setApproveStatus(approveStatus);
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                tPssrCircuitService.updateTPssrCircuit(item);
            }
        } else {
            lock.setSubId(tPssrCircuit.getSubId());
            lock.setApproveStatus(queryStatus);
            for (TPssrCircuit item : tPssrCircuitService.selectTPssrCircuitList(lock)) {
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                item.setApproveStatus(approveStatus);
                tPssrCircuitService.updateTPssrCircuit(item);
            }
        }
        //查询当前待审批的确认人
        TPssrCircuit entity = new TPssrCircuit();
        entity.setSubId(tPssrCircuit.getSubId());
        entity.setApproveStatus(queryStatus);
        for (TPssrCircuit item : tPssrCircuitService.selectTPssrCircuitList(entity)) {
            if (tPssrCircuit.getTaskType() == 4) {
                if (item.getConfirm1().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            } else if (tPssrCircuit.getTaskType() == 5) {
                if (item.getConfirm2().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }

        //无待审批任务结束当前用户流程

        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrCircuit.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();

    }

    /**
     * 驳回短接
     */
    @PutMapping("/turnDownCircuit")
    public AjaxResult turnDownCircuit(@RequestBody List<TPssrCircuit> tPssrCircuit) {
        if (CollectionUtils.isNotEmpty(tPssrCircuit)) {
            String userId = getUserId().toString();
            Long subId = tPssrCircuit.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrCircuit pssrCircuit : tPssrCircuit) {
                TPssrCircuit blind = new TPssrCircuit();
                blind.setId(pssrCircuit.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrCircuitService.updateTPssrCircuit(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(pssrCircuit.getSubId());
                turndown.setItemId(pssrCircuit.getId());
                turndown.setReason(pssrCircuit.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            ProcessEngine processEngine = ProcessEngines.getDefaultProcessEngine();
            TaskService taskService = processEngine.getTaskService();
            Task task = processEngine.getTaskService()//获取任务service
                    .createTaskQuery()//创建查询对象
                    .taskAssignee(userId)
                    .processInstanceId(approve.getProcessId()).singleResult();
            String taskId = task.getId();

            // 驳回 查询所有待审批的人员
            TPssrCircuit blind = new TPssrCircuit();
            blind.setSubId(subId);
            blind.setApproveStatus(1L);
            Set<String> installer = new HashSet<>();
            Set<String> remover = new HashSet<>();
            for (TPssrCircuit item : tPssrCircuitService.selectTPssrCircuitList(blind)) {

                // 安装人员
                installer.add(item.getConfirm1());

                //拆除人员
                remover.add(item.getConfirm2());
            }


            //处理流程节点
            Map<String, Object> param = new HashMap<>();
            param.put("condition", 1);
            param.put("confirmUsers1", new ArrayList<>(installer));
            param.put("confirmUsers2", new ArrayList<>(remover));
            //认领任务
            processEngine.getTaskService().claim(taskId, userId);
            taskService.addComment(taskId, approve.getProcessId(), "驳回");
            taskService.complete(taskId, param);

            // 修改审批表和sub表
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
