package com.ruoyi.project.training.newstaff.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.training.newstaff.domain.TTnFirstplan;
import com.ruoyi.project.training.newstaff.service.ITTnFirstplanService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 进组培训Controller
 *
 * @author ruoyi
 * @date 2023-07-04
 */
@RestController
@RequestMapping("/newstaff/firstplan")
public class TTnFirstplanController extends BaseController
{
    @Autowired
    private ITTnFirstplanService tTnFirstplanService;

    /**
     * 查询进组培训列表
     */
    @PreAuthorize("@ss.hasPermi('newstaff:firstplan:list')")
    @GetMapping("/list")
    public TableDataInfo list(TTnFirstplan tTnFirstplan)
    {
        startPage();
        List<TTnFirstplan> list = tTnFirstplanService.selectTTnFirstplanList(tTnFirstplan);
        return getDataTable(list);
    }

    /**
     * 导出进组培训列表
     */
    @PreAuthorize("@ss.hasPermi('newstaff:firstplan:export')")
    @Log(title = "进组培训", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TTnFirstplan tTnFirstplan)
    {
        List<TTnFirstplan> list = tTnFirstplanService.selectTTnFirstplanList(tTnFirstplan);
        ExcelUtil<TTnFirstplan> util = new ExcelUtil<TTnFirstplan>(TTnFirstplan.class);
        return util.exportExcel(list, "firstplan");
    }

    /**
     * 获取进组培训详细信息
     */
    @PreAuthorize("@ss.hasPermi('newstaff:firstplan:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tTnFirstplanService.selectTTnFirstplanById(id));
    }

    /**
     * 新增进组培训
     */
    @PreAuthorize("@ss.hasPermi('newstaff:firstplan:add')")
    @Log(title = "进组培训", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TTnFirstplan tTnFirstplan)
    {
        return toAjax(tTnFirstplanService.insertTTnFirstplan(tTnFirstplan));
    }

    /**
     * 修改进组培训
     */
    @PreAuthorize("@ss.hasPermi('newstaff:firstplan:edit')")
    @Log(title = "进组培训", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TTnFirstplan tTnFirstplan)
    {
        return toAjax(tTnFirstplanService.updateTTnFirstplan(tTnFirstplan));
    }

    /**
     * 修改进组培训
     */
    @PreAuthorize("@ss.hasPermi('newstaff:firstplan:edit')")
    @Log(title = "进组培训", businessType = BusinessType.UPDATE)
    @GetMapping("batchStatus/{ids}")
    public AjaxResult batchPlan(@PathVariable Long[] ids)
    {
        return toAjax(tTnFirstplanService.batchStatusTTnFirstplanByids(ids));
    }

    /**
     * 删除进组培训
     */
    @PreAuthorize("@ss.hasPermi('newstaff:firstplan:remove')")
    @Log(title = "进组培训", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tTnFirstplanService.deleteTTnFirstplanByIds(ids));
    }
}
