package com.ruoyi.project.training.newstaff.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.training.newstaff.domain.TTnTransferplan;
import com.ruoyi.project.training.newstaff.service.ITTnTransferplanService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 转岗培训Controller
 *
 * @author ruoyi
 * @date 2023-07-05
 */
@RestController
@RequestMapping("/newstaff/transferplan")
public class TTnTransferplanController extends BaseController
{
    @Autowired
    private ITTnTransferplanService tTnTransferplanService;

    /**
     * 查询转岗培训列表
     */
    @PreAuthorize("@ss.hasPermi('newstaff:transferplan:list')")
    @GetMapping("/list")
    public TableDataInfo list(TTnTransferplan tTnTransferplan)
    {
        startPage();
        List<TTnTransferplan> list = tTnTransferplanService.selectTTnTransferplanList(tTnTransferplan);
        return getDataTable(list);
    }

    /**
     * 导出转岗培训列表
     */
    @PreAuthorize("@ss.hasPermi('newstaff:transferplan:export')")
    @Log(title = "转岗培训", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TTnTransferplan tTnTransferplan)
    {
        List<TTnTransferplan> list = tTnTransferplanService.selectTTnTransferplanList(tTnTransferplan);
        ExcelUtil<TTnTransferplan> util = new ExcelUtil<TTnTransferplan>(TTnTransferplan.class);
        return util.exportExcel(list, "transferplan");
    }

    /**
     * 获取转岗培训详细信息
     */
    @PreAuthorize("@ss.hasPermi('newstaff:transferplan:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tTnTransferplanService.selectTTnTransferplanById(id));
    }

    /**
     * 新增转岗培训
     */
    @PreAuthorize("@ss.hasPermi('newstaff:transferplan:add')")
    @Log(title = "转岗培训", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TTnTransferplan tTnTransferplan)
    {
        return toAjax(tTnTransferplanService.insertTTnTransferplan(tTnTransferplan));
    }

    /**
     * 修改转岗培训
     */
    @PreAuthorize("@ss.hasPermi('newstaff:transferplan:edit')")
    @Log(title = "转岗培训", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TTnTransferplan tTnTransferplan)
    {
        return toAjax(tTnTransferplanService.updateTTnTransferplan(tTnTransferplan));
    }

    /**
     * 删除转岗培训
     */
    @PreAuthorize("@ss.hasPermi('newstaff:transferplan:remove')")
    @Log(title = "转岗培训批量", businessType = BusinessType.UPDATE)
    @GetMapping("batchStatus/{ids}")
    public AjaxResult batchPlan(@PathVariable Long[] ids)
    {
        return toAjax(tTnTransferplanService.batchStatusByIds(ids));
    }

    /**
     * 删除转岗培训
     */
    @PreAuthorize("@ss.hasPermi('newstaff:transferplan:remove')")
    @Log(title = "转岗培训", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tTnTransferplanService.deleteTTnTransferplanByIds(ids));
    }
}
