import Vue from 'vue'
import Cookies from 'js-cookie'

import 'normalize.css/normalize.css' // a modern alternative to CSS resets

import Element from 'element-ui'
import './assets/styles/element-variables.scss'
import '@/assets/styles/index.scss' // global css
import '@/assets/styles/ruoyi.scss' // ruoyi css


import App from './App'
import store from './store'
import router from './router'
import permission from './directive/permission'
import '@/style/common.scss'
import '@/assets/icons' // icon
import './permission' // permission control
import { getDicts } from "@/api/system/dict/data";
import { getConfigKey } from "@/api/system/config";
import { parseTime, resetForm, addDateRange, selectDictLabel, selectDictLabels, download, handleTree } from "@/utils/ruoyi";
import Pagination from "@/components/Pagination";
import i18n from './lang'
import "@/components/dialogdrag.js"
import dataV from '@jiaminghi/data-view';
//自定义表格工具扩展
import RightToolbar from "@/components/RightToolbar"
//echarts
import echarts from '@/api/cpms/echarts.min.js'
import officeConvert from '@/utils/officeConvert.js'  //全局预览方法
//import { Loading } from 'element-ui'; //全局loading
// 按需引入vue-awesome图标
import Icon from 'vue-awesome/components/Icon';
import 'vue-awesome/icons/chart-bar.js';
import 'vue-awesome/icons/chart-area.js';
import 'vue-awesome/icons/chart-pie.js';
import 'vue-awesome/icons/chart-line.js';
import 'vue-awesome/icons/align-left.js';
import '@/assets/iconfont/iconfont.css'
import '@/assets/iconfont/iconfont.js'
// 适配flex
import { UTable, UTableColumn } from 'umy-ui';
// 字典标签组件
import DictTag from '@/components/DictTag'
// 字典数据组件
import DictData from '@/components/DictData'

// 函数节流
import throttle from "@/utils/throttle";
import {download2} from "@/utils/request";
Vue.prototype.$throttle = throttle;


// 全局注册图标
Vue.component('icon', Icon);
// 全局方法挂载
Vue.prototype.bus = new Vue()
Vue.prototype.getDicts = getDicts
Vue.prototype.getConfigKey = getConfigKey
Vue.prototype.parseTime = parseTime
Vue.prototype.resetForm = resetForm
Vue.prototype.addDateRange = addDateRange
Vue.prototype.selectDictLabel = selectDictLabel
Vue.prototype.selectDictLabels = selectDictLabels
Vue.prototype.download = download
Vue.prototype.download2 = download2
Vue.prototype.handleTree = handleTree
Vue.prototype.echarts = echarts
Vue.prototype.officeConvert = officeConvert


Vue.prototype.msgSuccess = function (msg) {
  this.$message({ showClose: true, message: msg, type: "success" });
}

Vue.prototype.msgError = function (msg) {
  this.$message({ showClose: true, message: msg, type: "error" });
}

Vue.prototype.msgInfo = function (msg) {
  this.$message.info(msg);
}

// 全局组件挂载
Vue.component('DictTag', DictTag)
Vue.component('Pagination', Pagination)
Vue.component('RightToolbar', RightToolbar)

Vue.use(permission)
Vue.use(dataV)
Vue.use(UTable)
Vue.use(UTableColumn)
DictData.install()
/**
 * If you don't want to use mock-server
 * you want to use MockJs for mock api
 * you can execute: mockXHR()
 *
 * Currently MockJs will be used in the production environment,
 * please remove it before going online! ! !
 */

Vue.use(Element, {
  size: 'mini', // set element-ui default size
  i18n: (key, value) => i18n.t(key, value)
})

Vue.config.productionTip = false

new Vue({
  el: '#app',
  router,
  store,
  i18n,
  render: h => h(App)
})
